Index: channels/chan_iax2.c
===================================================================
--- channels/chan_iax2.c	(revision 358857)
+++ channels/chan_iax2.c	(working copy)
@@ -5664,6 +5664,8 @@
 		break;
 	}
 
+	ast_verb(1, "Sending %p %d\n", data, (int) datalen);
+
 	res = send_command(pvt, AST_FRAME_CONTROL, condition, 0, data, datalen, -1);
 
 done:
@@ -7158,6 +7160,46 @@
 	return 0;
 }
 
+static char *handle_cli_iax2_hold_channel(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
+{
+	struct ast_channel *c;
+
+	switch (cmd) {
+	case CLI_INIT:
+		e->command = "iax2 {hold|unhold|kill}";
+		e->usage =
+			"Usage: iax2 {hold|unhold|kill} <channel>\n"
+			"       Indicates to the underlying channel that it is holding or\n"
+			"       has been released from hold.\n";
+		return NULL;
+	case CLI_GENERATE:
+		return ast_complete_channels(a->line, a->word, a->pos, a->n, 2);
+	}
+
+	if (a->argc != 3) {
+		return CLI_SHOWUSAGE;
+	}
+
+	if (!(c = ast_channel_get_by_name(a->argv[2]))) {
+		ast_cli(a->fd, "%s is not a known channel\n", a->argv[2]);
+		return CLI_SUCCESS;
+	}
+
+	ast_channel_lock(c);
+
+	if (a->argv[1][0] == 'h') {
+		ast_indicate_data(c, AST_CONTROL_HOLD, "default", sizeof("default"));
+	} else if (a->argv[1][0] == 'k') {
+		ast_indicate(c, AST_CONTROL_HOLD);
+	} else {
+		ast_indicate(c, AST_CONTROL_UNHOLD);
+	}
+
+	ast_channel_unlock(c);
+
+	return CLI_SUCCESS;
+}
+
 static char *handle_cli_iax2_show_channels(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
 #define FORMAT2 "%-20.20s  %-15.15s  %-10.10s  %-11.11s  %-11.11s  %-7.7s  %-6.6s  %-6.6s  %s  %s  %9s\n"
@@ -14157,6 +14199,7 @@
 	AST_CLI_DEFINE(handle_cli_iax2_test_losspct,        "Set IAX2 incoming frame loss percentage"),
 	AST_CLI_DEFINE(handle_cli_iax2_unregister,          "Unregister (force expiration) an IAX2 peer from the registry"),
 	AST_CLI_DEFINE(handle_cli_iax2_show_callno_limits,  "Show current entries in IP call number limit table"),
+	AST_CLI_DEFINE(handle_cli_iax2_hold_channel,        "Hold/unhold the specified channel"),
 #ifdef IAXTESTS
 	AST_CLI_DEFINE(handle_cli_iax2_test_jitter,         "Simulates jitter for testing"),
 	AST_CLI_DEFINE(handle_cli_iax2_test_late,           "Test the receipt of a late frame"),
